package org.choub.minecraft.launcher.task;

import java.text.MessageFormat;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.ThreadPoolExecutor;

import org.apache.log4j.Logger;
import org.choub.minecraft.launcher.LauncherContext;


public final class TaskManager {

	private static Logger logger = Logger.getLogger(TaskManager.class);
	private static final int MAX_CONCURRENT_THREAD = 255;

	private Object lock = new Object();
	private ScheduledThreadPoolExecutor executor;
	private LauncherContext context;

	public TaskManager(LauncherContext context) {
		this.context = context;
		createExecutor();
	}

	protected void createExecutor() {
		executor = new ScheduledThreadPoolExecutor(MAX_CONCURRENT_THREAD, new ThreadPoolExecutor.DiscardOldestPolicy());
	}

	public void cancelAllTasks() {
		if (executor != null) {
			logger.info("Task Manager is cancelling all tasks...");
			executor.shutdownNow();
			logger.info(MessageFormat.format("[{0}] task(s) remaining.", executor.getActiveCount()));
		}
	}

	public void reload() {
		synchronized (lock) {
			cancelAllTasks();
			logger.info("Internal Scheduler is reloading ...");
			createExecutor();
		}
	}

	public void run(BackgroundTask task) {
		synchronized (lock) {
			if (executor == null) {
				throw new IllegalStateException("The Task Manager is not ready. Its timer must be initialized");
			}

			task.setContext(context);
			if (executor.isShutdown()) {
				reload();
			}
			executor.execute(task);
		}
	}
}
